@@ -1,4 +1,4 @@
-/* 
+/*
  * Scythe Statistical Library Copyright (C) 2000-2002 Andrew D. Martin
  * and Kevin M. Quinn; 2002-present Andrew D. Martin, Kevin M. Quinn,
  * and Daniel Pemstein.  All Rights Reserved.
@@ -16,7 +16,7 @@
  * in rng.cc is based on that in the R project, version 1.6.0-1.7.1.
  * This code is available under the terms of the GNU GPL.  Original
  * copyright:
- * 
+ *
  * Copyright (C) 1998      Ross Ihaka
  * Copyright (C) 2000-2002 The R Development Core Team
  * Copyright (C) 2003      The R Foundation
@@ -59,15 +59,12 @@
 #include "scythestat/la.h"
 #endif
 
-#include <R.h>           // needed to use Rprintf()
-#include <R_ext/Utils.h> // needed to allow user interrupts
-
 namespace scythe {
 
 /* Shorthand for the matrix versions of the various distributions'
  * random number generators.
  */
-  
+
 #define SCYTHE_RNGMETH_MATRIX(NAME, RTYPE, ARGNAMES, ...)             \
   template <matrix_order O, matrix_style S>                           \
   Matrix<RTYPE, O, S>                                                 \
@@ -170,8 +167,8 @@ namespace scythe {
        * is illegal to make template methods virtual
        */
       template <matrix_order O, matrix_style S>
-      Matrix<double,O,S> runif(unsigned int rows, 
-                               unsigned int cols) 
+      Matrix<double,O,S> runif(unsigned int rows,
+                               unsigned int cols)
       {
         Matrix<double, O, S> ret(rows, cols, false);
         typename Matrix<double,O,S>::forward_iterator it;
@@ -182,7 +179,7 @@ namespace scythe {
         return ret;
       }
 
-      Matrix<double,Col,Concrete> runif(unsigned int rows, 
+      Matrix<double,Col,Concrete> runif(unsigned int rows,
                                         unsigned int cols)
       {
         return runif<Col,Concrete>(rows, cols);
@@ -196,7 +193,7 @@ namespace scythe {
        *
        * \param alpha The first positive beta shape parameter.
        * \param beta the second positive beta shape parameter.
-			 * 
+			 *
 			 * \see pbeta(double x, double a, double b)
 			 * \see dbeta(double x, double a, double b)
 			 * \see betafn(double a, double b)
@@ -209,15 +206,15 @@ namespace scythe {
       {
         double report;
         double xalpha, xbeta;
-        
+
         // Check for allowable parameters
         SCYTHE_CHECK_10(alpha <= 0, scythe_invalid_arg, "alpha <= 0");
         SCYTHE_CHECK_10(beta <= 0, scythe_invalid_arg, "beta <= 0");
-        
+
         xalpha = rchisq (2 * alpha);
         xbeta = rchisq (2 * beta);
         report = xalpha / (xalpha + xbeta);
-        
+
         return (report);
       }
 
@@ -240,7 +237,7 @@ namespace scythe {
        *
        * \throw scythe_convergence_error (Level 0)
        */
-      double 
+      double
       rnchypgeom(double m1, double n1, double n2, double psi,
                  double delta)
       {
@@ -248,7 +245,7 @@ namespace scythe {
         double a = psi - 1;
         double b = -1 * ((n1+m1+2)*psi + n2 - m1);
         double c = psi * (n1+1) * (m1+1);
-        double q = -0.5 * ( b + sgn(b) * 
+        double q = -0.5 * ( b + sgn(b) *
             std::sqrt(std::pow(b,2) - 4*a*c));
         double root1 = c/q;
         double root2 = q/a;
@@ -260,7 +257,7 @@ namespace scythe {
           mode = std::floor(root2);
           exactcheck = 1;
         }
-     
+
 
         int size = static_cast<int>(u+1);
 
@@ -268,7 +265,7 @@ namespace scythe {
         fvec[static_cast<int>(mode)] = 1.0;
         double s;
         // compute the mass function at y
-        if (delta <= 0 || exactcheck==1){  //exact evaluation 
+        if (delta <= 0 || exactcheck==1){  //exact evaluation
           // sum from mode to u
           double f = 1.0;
           s = 1.0;
@@ -278,7 +275,7 @@ namespace scythe {
             s += f;
             fvec[static_cast<int>(i)] = f;
           }
-         
+
           // sum from mode to el
           f = 1.0;
           for (double i=(mode-1); i>=el; --i){
@@ -302,7 +299,7 @@ namespace scythe {
             fvec[static_cast<int>(i)] = f;
             ++i;
           } while(f>=epsilon || r>=5.0/6.0);
-         
+
           // sum from mode to elstar
           f = 1.0;
           i = mode-1;
@@ -313,7 +310,7 @@ namespace scythe {
             s += f;
             fvec[static_cast<int>(i)] = f;
             --i;
-          } while(f>=epsilon || r <=6.0/5.0);         
+          } while(f>=epsilon || r <=6.0/5.0);
         }
 
         double udraw = runif();
@@ -328,7 +325,7 @@ namespace scythe {
           double fu;
           if (lower >= el)
             fl = fvec[static_cast<int>(lower)];
-          else 
+          else
             fl = 0.0;
 
           if (upper <= u)
@@ -348,7 +345,7 @@ namespace scythe {
             ++upper;
           }
         } while(udraw>psum);
-       
+
         delete [] fvec;
         SCYTHE_THROW(scythe_convergence_error,
           "Algorithm did not converge");
@@ -364,7 +361,7 @@ namespace scythe {
 			 * Bernoulli distribution with probability of success \a p.
        *
        * \param p The probability of success on a trial.
-			 * 
+			 *
        * \throw scythe_invalid_arg (Level 1)
        */
       unsigned int
@@ -372,22 +369,22 @@ namespace scythe {
       {
         unsigned int report;
         double unif;
-          
+
         // Check for allowable paramters
         SCYTHE_CHECK_10(p < 0 || p > 1, scythe_invalid_arg,
             "p parameter not in[0,1]");
-        
+
         unif = runif ();
         if (unif < p)
           report = 1;
         else
           report = 0;
-        
+
         return (report);
       }
 
       SCYTHE_RNGMETH_MATRIX(rbern, unsigned int, p, double p);
-      
+
       /*! \brief Generate a binomial distributed random variate.
        *
 			 * This function returns a pseudo-random variate drawn from the
@@ -396,7 +393,7 @@ namespace scythe {
        *
        * \param n The number of trials.
        * \param p The probability of success on each trial.
-			 * 
+			 *
 			 * \see pbinom(double x, unsigned int n, double p)
 			 * \see dbinom(double x, unsigned int n, double p)
 			 *
@@ -408,12 +405,12 @@ namespace scythe {
         unsigned int report;
         unsigned int count = 0;
         double hold;
-          
+
         // Check for allowable parameters
         SCYTHE_CHECK_10(n == 0, scythe_invalid_arg, "n == 0");
-        SCYTHE_CHECK_10(p < 0 || p > 1, scythe_invalid_arg, 
+        SCYTHE_CHECK_10(p < 0 || p > 1, scythe_invalid_arg,
             "p not in [0,1]");
-          
+
         // Loop and count successes
         for (unsigned int i = 0; i < n; i++) {
           hold = runif ();
@@ -421,7 +418,7 @@ namespace scythe {
             ++count;
         }
         report = count;
-        
+
         return (report);
       }
 
@@ -434,7 +431,7 @@ namespace scythe {
 			 * \f$\chi^2\f$distribution with \a df degress of freedom.
        *
        * \param df The degrees of freedom.
-			 * 
+			 *
 			 * \see pchisq(double x, double df)
 			 * \see dchisq(double x, double df)
 			 *
@@ -444,14 +441,14 @@ namespace scythe {
       rchisq (double df)
       {
         double report;
-          
+
         // Check for allowable paramter
         SCYTHE_CHECK_10(df <= 0, scythe_invalid_arg,
             "Degrees of freedom <= 0");
-      
+
         // Return Gamma(nu/2, 1/2) variate
         report = rgamma (df / 2, .5);
-        
+
         return (report);
       }
 
@@ -464,7 +461,7 @@ namespace scythe {
 			 * parameter \a invscale.
        *
        * \param invscale The inverse scale parameter.
-			 * 
+			 *
 			 * \see pexp(double x, double scale)
 			 * \see dexp(double x, double scale)
 			 *
@@ -474,18 +471,18 @@ namespace scythe {
       rexp (double invscale)
       {
         double report;
-        
+
         // Check for allowable parameter
         SCYTHE_CHECK_10(invscale <= 0, scythe_invalid_arg,
             "Inverse scale parameter <= 0");
-        
+
         report = -std::log (runif ()) / invscale;
-        
+
         return (report);
       }
 
       SCYTHE_RNGMETH_MATRIX(rexp, double, invscale, double invscale);
-    
+
       /*! \brief Generate an F distributed random variate.
        *
 			 * This function returns a pseudo-random variate drawn from the
@@ -520,7 +517,7 @@ namespace scythe {
        *
        * \param shape The strictly positive shape of the distribution.
 			 * \param rate The inverse of the strictly positive scale of the distribution.  That is, 1 / scale.
-			 * 
+			 *
 			 * \see pgamma(double x, double shape, double scale)
 			 * \see dgamma(double x, double shape, double scale)
 			 * \see gammafn(double x)
@@ -542,7 +539,7 @@ namespace scythe {
         else if (shape == 1)
           report = -std::log (runif ()) / rate;
         else
-          report = rgamma1 (shape + 1) 
+          report = rgamma1 (shape + 1)
             * std::pow (runif (), 1 / shape) / rate;
 
         return (report);
@@ -559,7 +556,7 @@ namespace scythe {
        *
        * \param location The location of the distribution.
 			 * \param scale The scale of the distribution.
-			 * 
+			 *
 			 * \see plogis(double x, double location, double scale)
 			 * \see dlogis(double x, double location, double scale)
 			 *
@@ -570,17 +567,17 @@ namespace scythe {
       {
         double report;
         double unif;
-          
+
         // Check for allowable paramters
         SCYTHE_CHECK_10(scale <= 0, scythe_invalid_arg, "scale <= 0");
-        
+
         unif = runif ();
         report = location + scale * std::log (unif / (1 - unif));
-        
+
         return (report);
       }
 
-      SCYTHE_RNGMETH_MATRIX(rlogis, double, 
+      SCYTHE_RNGMETH_MATRIX(rlogis, double,
           SCYTHE_ARGSET(location, scale),
           double location, double scale);
 
@@ -593,7 +590,7 @@ namespace scythe {
        * \param logmean The logged mean of the distribtion.
 			 * \param logsd The strictly positive logged standard deviation
 			 * of the distribution.
-			 * 
+			 *
 			 * \see plnorm(double x, double logmean, double logsd)
 			 * \see dlnorm(double x, double logmean, double logsd)
 			 *
@@ -608,7 +605,7 @@ namespace scythe {
         return std::exp(rnorm(logmean, logsd));
       }
 
-      SCYTHE_RNGMETH_MATRIX(rlnorm, double, 
+      SCYTHE_RNGMETH_MATRIX(rlnorm, double,
           SCYTHE_ARGSET(logmean, logsd),
           double logmean, double logsd);
 
@@ -622,7 +619,7 @@ namespace scythe {
        * \param n The strictly positive target number of successful
 			 * trials (dispersion parameters).
 			 * \param p The probability of success on each trial.
-			 * 
+			 *
 			 * \see pnbinom(unsigned int x, double n, double p)
 			 * \see dnbinom(unsigned int x, double n, double p)
 			 *
@@ -634,7 +631,7 @@ namespace scythe {
         SCYTHE_CHECK_10(n == 0 || p <= 0 || p > 1, scythe_invalid_arg,
             "n == 0, p <= 0, or p > 1");
 
-        return rpois(rgamma(n, (1 - p) / p));
+        return rpois(rgamma(n, p / (1 - p)));
       }
 
       SCYTHE_RNGMETH_MATRIX(rnbinom, unsigned int,
@@ -648,7 +645,7 @@ namespace scythe {
        *
        * \param mean The mean of the distribution.
 			 * \param sd The standard deviation of the distribution.
-			 * 
+			 *
 			 * \see pnorm(double x, double mean, double sd)
 			 * \see dnorm(double x, double mean, double sd)
 			 *
@@ -657,9 +654,9 @@ namespace scythe {
       double
       rnorm (double mean = 0, double sd = 1)
       {
-        SCYTHE_CHECK_10(sd <= 0, scythe_invalid_arg, 
+        SCYTHE_CHECK_10(sd <= 0, scythe_invalid_arg,
             "Negative standard deviation");
-        
+
         return (mean + rnorm1 () * sd);
       }
 
@@ -674,7 +671,7 @@ namespace scythe {
        *
        * \param lambda The strictly positive expected number of
 			 * occurrences.
-			 * 
+			 *
 			 * \see ppois(double x, double lambda)
 			 * \see dpois(double x, double lambda)
 			 *
@@ -685,7 +682,7 @@ namespace scythe {
       {
         SCYTHE_CHECK_10(lambda <= 0, scythe_invalid_arg, "lambda <= 0");
         unsigned int n;
-        
+
         if (lambda < 33) {
           double cutoff = std::exp(-lambda);
           n = -1;
@@ -693,21 +690,21 @@ namespace scythe {
           do {
             ++n;
             t *= runif();
-          } while (t > cutoff);    
+          } while (t > cutoff);
         } else {
           bool accept = false;
           double c = 0.767 - 3.36/lambda;
           double beta = M_PI/std::sqrt(3*lambda);
           double alpha = lambda*beta;
           double k = std::log(c) - lambda - std::log(beta);
-            
+
           while (! accept){
             double u1 = runif();
             double x = (alpha - std::log((1-u1)/u1))/beta;
             while (x <= -0.5){
               u1 = runif();
               x = (alpha - std::log((1-u1)/u1))/beta;
-            } 
+            }
             n = static_cast<int>(x + 0.5);
             double u2 = runif();
             double lhs = alpha - beta*x +
@@ -717,7 +714,7 @@ namespace scythe {
               accept = true;
           }
         }
-        
+
         return n;
       }
 
@@ -738,7 +735,7 @@ namespace scythe {
        * \param mu The mean of the distribution.
 			 * \param sigma2 The variance of the distribution.
 			 * \param nu The degrees of freedom of the distribution.
-			 * 
+			 *
 			 * \see dt1(double x, double mu, double sigma2, double nu)
 			 *
        * \throw scythe_invalid_arg (Level 1)
@@ -748,18 +745,18 @@ namespace scythe {
       {
         double report;
         double x, z;
-          
+
         // Check for allowable paramters
         SCYTHE_CHECK_10(sigma2 <= 0, scythe_invalid_arg,
             "Variance parameter sigma2 <= 0");
         SCYTHE_CHECK_10(nu <= 0, scythe_invalid_arg,
             "D.O.F parameter nu <= 0");
-        
+
         z = rnorm1 ();
         x = rchisq (nu);
-        report = mu + std::sqrt (sigma2) * z 
+        report = mu + std::sqrt (sigma2) * z
           * std::sqrt (nu) / std::sqrt (x);
-        
+
         return (report);
       }
 
@@ -773,7 +770,7 @@ namespace scythe {
        *
        * \param shape The strictly positive shape of the distribution.
 			 * \param scale The strictly positive scale of the distribution.
-			 * 
+			 *
 			 * \see pweibull(double x, double shape, double scale)
 			 * \see dweibull(double x, double shape, double scale)
 			 *
@@ -808,11 +805,11 @@ namespace scythe {
       richisq (double nu)
       {
         double report;
-          
+
         // Check for allowable parameter
         SCYTHE_CHECK_10(nu <= 0, scythe_invalid_arg,
             "Degrees of freedom <= 0");
-          
+
         // Return Inverse-Gamma(nu/2, 1/2) variate
         report = rigamma (nu / 2, .5);
         return (report);
@@ -827,7 +824,7 @@ namespace scythe {
        *
        * \param shape The strictly positive shape of the distribution.
 			 * \param scale The strictly positive scale of the distribution.
-			 * 
+			 *
 			 * \see rgamma(double alpha, double beta)
 			 *
        * \throw scythe_invalid_arg (Level 1)
@@ -836,7 +833,7 @@ namespace scythe {
       rigamma (double alpha, double beta)
       {
         double report;
-        
+
         // Check for allowable parameters
         SCYTHE_CHECK_10(alpha <= 0, scythe_invalid_arg, "alpha <= 0");
         SCYTHE_CHECK_10(beta <= 0, scythe_invalid_arg, "beta <= 0");
@@ -864,7 +861,7 @@ namespace scythe {
 			 * \param variance The variance of the distribution.
 			 * \param below The lower truncation point of the distribution.
 			 * \param above The upper truncation point of the distribution.
-			 * 
+			 *
 			 * \see rtnorm_combo(double mean, double variance, double below, double above)
 			 * \see rtbnorm_slice(double mean, double variance, double below, unsigned int iter = 10)
 			 * \see rtanorm_slice(double mean, double variance, double above, unsigned int iter = 10)
@@ -874,30 +871,30 @@ namespace scythe {
 			 *
        * \throw scythe_invalid_arg (Level 1)
        */
-      double 
+      double
       rtnorm(double mean, double variance, double below, double above)
-      {  
+      {
         SCYTHE_CHECK_10(below >= above, scythe_invalid_arg,
             "Truncation bound not logically consistent");
         SCYTHE_CHECK_10(variance <= 0, scythe_invalid_arg,
             "Variance <= 0");
-        
+
         double sd = std::sqrt(variance);
         double FA = 0.0;
         double FB = 0.0;
-        if ((std::fabs((above-mean)/sd) < 8.2) 
+        if ((std::fabs((above-mean)/sd) < 8.2)
             && (std::fabs((below-mean)/sd) < 8.2)){
           FA = pnorm1((above-mean)/sd, true, false);
           FB = pnorm1((below-mean)/sd, true, false);
         }
-        if ((((above-mean)/sd) < 8.2)  && (((below-mean)/sd) <= -8.2) ){ 
+        if ((((above-mean)/sd) < 8.2)  && (((below-mean)/sd) <= -8.2) ){
           FA = pnorm1((above-mean)/sd, true, false);
           FB = 0.0;
         }
-        if ( (((above-mean)/sd) >= 8.2)  && (((below-mean)/sd) > -8.2) ){ 
+        if ( (((above-mean)/sd) >= 8.2)  && (((below-mean)/sd) > -8.2) ){
           FA = 1.0;
           FB = pnorm1((below-mean)/sd, true, false);
-        } 
+        }
         if ( (((above-mean)/sd) >= 8.2) && (((below-mean)/sd) <= -8.2)){
           FA = 1.0;
           FB = 0.0;
@@ -912,11 +909,11 @@ namespace scythe {
           draw = above;
         if (draw < below)
           draw = below;
-         
+
         return draw;
       }
 
-      SCYTHE_RNGMETH_MATRIX(rtnorm, double, 
+      SCYTHE_RNGMETH_MATRIX(rtnorm, double,
           SCYTHE_ARGSET(mean, variance, above, below), double mean,
           double variance, double above, double below);
 
@@ -934,7 +931,7 @@ namespace scythe {
 			 * \param variance The variance of the distribution.
 			 * \param below The lower truncation point of the distribution.
 			 * \param above The upper truncation point of the distribution.
-			 * 
+			 *
 			 * \see rtnorm(double mean, double variance, double below, double above)
 			 * \see rtbnorm_slice(double mean, double variance, double below, unsigned int iter = 10)
 			 * \see rtanorm_slice(double mean, double variance, double above, unsigned int iter = 10)
@@ -945,20 +942,20 @@ namespace scythe {
        * \throw scythe_invalid_arg (Level 1)
        */
       double
-      rtnorm_combo(double mean, double variance, double below, 
+      rtnorm_combo(double mean, double variance, double below,
                    double above)
       {
         SCYTHE_CHECK_10(below >= above, scythe_invalid_arg,
             "Truncation bound not logically consistent");
         SCYTHE_CHECK_10(variance <= 0, scythe_invalid_arg,
             "Variance <= 0");
-        
+
         double sd = std::sqrt(variance);
         if ((((above-mean)/sd > 0.5) && ((mean-below)/sd > 0.5))
             ||
             (((above-mean)/sd > 2.0) && ((below-mean)/sd < 0.25))
             ||
-            (((mean-below)/sd > 2.0) && ((above-mean)/sd > -0.25))) { 
+            (((mean-below)/sd > 2.0) && ((above-mean)/sd > -0.25))) {
           double x = rnorm(mean, sd);
           while ((x > above) || (x < below))
             x = rnorm(mean,sd);
@@ -967,19 +964,19 @@ namespace scythe {
           // use the inverse cdf method
           double FA = 0.0;
           double FB = 0.0;
-          if ((std::fabs((above-mean)/sd) < 8.2) 
+          if ((std::fabs((above-mean)/sd) < 8.2)
               && (std::fabs((below-mean)/sd) < 8.2)){
             FA = pnorm1((above-mean)/sd, true, false);
             FB = pnorm1((below-mean)/sd, true, false);
           }
-          if ((((above-mean)/sd) < 8.2)  && (((below-mean)/sd) <= -8.2) ){ 
+          if ((((above-mean)/sd) < 8.2)  && (((below-mean)/sd) <= -8.2) ){
             FA = pnorm1((above-mean)/sd, true, false);
             FB = 0.0;
           }
-          if ( (((above-mean)/sd) >= 8.2)  && (((below-mean)/sd) > -8.2) ){ 
+          if ( (((above-mean)/sd) >= 8.2)  && (((below-mean)/sd) > -8.2) ){
             FA = 1.0;
             FB = pnorm1((below-mean)/sd, true, false);
-          } 
+          }
           if ( (((above-mean)/sd) >= 8.2) && (((below-mean)/sd) <= -8.2)){
             FA = 1.0;
             FB = 0.0;
@@ -995,10 +992,10 @@ namespace scythe {
           if (x < below)
             x = below;
           return x;
-        }    
+        }
       }
 
-      SCYTHE_RNGMETH_MATRIX(rtnorm_combo, double, 
+      SCYTHE_RNGMETH_MATRIX(rtnorm_combo, double,
           SCYTHE_ARGSET(mean, variance, above, below), double mean,
           double variance, double above, double below);
 
@@ -1014,7 +1011,7 @@ namespace scythe {
 			 * \param variance The variance of the distribution.
 			 * \param below The lower truncation point of the distribution.
 			 * \param iter The number of iterations to use.
-			 * 
+			 *
 			 * \see rtnorm(double mean, double variance, double below, double above)
 			 * \see rtnorm_combo(double mean, double variance, double below, double above)
 			 * \see rtanorm_slice(double mean, double variance, double above, unsigned int iter = 10)
@@ -1032,26 +1029,27 @@ namespace scythe {
             "Truncation point < mean");
         SCYTHE_CHECK_10(variance <= 0, scythe_invalid_arg,
             "Variance <= 0");
-         
+
         double z = 0;
         double x = below + .00001;
-         
+
         for (unsigned int i=0; i<iter; ++i){
           z = runif()*std::exp(-1*std::pow((x-mean),2)/(2*variance));
           x = runif()*
             ((mean + std::sqrt(-2*variance*std::log(z))) - below) + below;
         }
 
-        if (! finite(x)) {
-          Rprintf("Mean extremely far from truncation point. Returning truncation point");
-          return below; 
+        if (! R_finite(x)) {
+          SCYTHE_WARN("Mean extremely far from truncation point. "
+              << "Returning truncation point");
+          return below;
         }
 
         return x;
       }
 
-      SCYTHE_RNGMETH_MATRIX(rtbnorm_slice, double, 
-          SCYTHE_ARGSET(mean, variance, below, iter), double mean, 
+      SCYTHE_RNGMETH_MATRIX(rtbnorm_slice, double,
+          SCYTHE_ARGSET(mean, variance, below, iter), double mean,
           double variance, double below, unsigned int iter = 10);
 
 			/*! \brief Generate a normally distributed random variate,
@@ -1066,7 +1064,7 @@ namespace scythe {
 			 * \param variance The variance of the distribution.
 			 * \param above The upper truncation point of the distribution.
 			 * \param iter The number of iterations to use.
-			 * 
+			 *
 			 * \see rtnorm(double mean, double variance, double below, double above)
 			 * \see rtnorm_combo(double mean, double variance, double below, double above)
 			 * \see rtbnorm_slice(double mean, double variance, double below, unsigned int iter = 10)
@@ -1077,36 +1075,37 @@ namespace scythe {
        * \throw scythe_invalid_arg (Level 1)
        */
       double
-      rtanorm_slice (double mean, double variance, double above, 
+      rtanorm_slice (double mean, double variance, double above,
           unsigned int iter = 10)
       {
         SCYTHE_CHECK_10(above > mean, scythe_invalid_arg,
             "Truncation point > mean");
         SCYTHE_CHECK_10(variance <= 0, scythe_invalid_arg,
             "Variance <= 0");
-      
+
         double below = -1*above;
         double newmu = -1*mean;
         double z = 0;
         double x = below + .00001;
-         
+
         for (unsigned int i=0; i<iter; ++i){
           z = runif()*std::exp(-1*std::pow((x-newmu),2)
               /(2*variance));
           x = runif()
-            *( (newmu + std::sqrt(-2*variance*std::log(z))) - below) 
+            *( (newmu + std::sqrt(-2*variance*std::log(z))) - below)
             + below;
         }
-        if (! finite(x)) {
-          Rprintf("Mean extremely far from truncation point. Returning truncation point");
-          return above; 
+        if (! R_finite(x)) {
+          SCYTHE_WARN("Mean extremely far from truncation point. "
+              << "Returning truncation point");
+          return above;
         }
-        
+
         return -1*x;
       }
 
-      SCYTHE_RNGMETH_MATRIX(rtanorm_slice, double, 
-          SCYTHE_ARGSET(mean, variance, above, iter), double mean, 
+      SCYTHE_RNGMETH_MATRIX(rtanorm_slice, double,
+          SCYTHE_ARGSET(mean, variance, above, iter), double mean,
           double variance, double above, unsigned int iter = 10);
 
 			/*! \brief Generate a normally distributed random
@@ -1124,7 +1123,7 @@ namespace scythe {
 			 * \param below The lower truncation point of the distribution.
 			 * \param iter The number of iterations to run the slice
 			 * sampler.
-			 * 
+			 *
 			 * \see rtnorm(double mean, double variance, double below, double above)
 			 * \see rtnorm_combo(double mean, double variance, double below, double above)
 			 * \see rtbnorm_slice(double mean, double variance, double below, unsigned int iter = 10)
@@ -1135,12 +1134,12 @@ namespace scythe {
        * \throw scythe_invalid_arg (Level 1)
        */
       double
-      rtbnorm_combo (double mean, double variance, double below, 
+      rtbnorm_combo (double mean, double variance, double below,
           unsigned int iter = 10)
       {
         SCYTHE_CHECK_10(variance <= 0, scythe_invalid_arg,
             "Variance <= 0");
-        
+
         double s = std::sqrt(variance);
         // do rejection sampling and return value
         //if (m >= below){
@@ -1148,7 +1147,7 @@ namespace scythe {
           double x = rnorm(mean, s);
           while (x < below)
             x = rnorm(mean,s);
-          return x; 
+          return x;
         } else if ((mean/s - below/s ) > -5.0 ){
           // use the inverse cdf method
           double above =  std::numeric_limits<double>::infinity();
@@ -1161,20 +1160,21 @@ namespace scythe {
           for (unsigned int i=0; i<iter; ++i){
             z = runif() * std::exp(-1 * std::pow((x - mean), 2)
                 / (2 * variance));
-            x = runif() 
-              * ((mean + std::sqrt(-2 * variance * std::log(z))) 
+            x = runif()
+              * ((mean + std::sqrt(-2 * variance * std::log(z)))
                 - below) + below;
           }
-          if (! finite(x)) {
-            Rprintf("Mean extremely far from truncation point. Returning truncation point");
-            return below; 
+          if (! R_finite(x)) {
+            SCYTHE_WARN("Mean extremely far from truncation point. "
+                << "Returning truncation point");
+            return below;
           }
           return x;
         }
       }
 
-      SCYTHE_RNGMETH_MATRIX(rtbnorm_combo, double, 
-          SCYTHE_ARGSET(mean, variance, below, iter), double mean, 
+      SCYTHE_RNGMETH_MATRIX(rtbnorm_combo, double,
+          SCYTHE_ARGSET(mean, variance, below, iter), double mean,
           double variance, double below, unsigned int iter = 10);
 
 			/*! \brief Generate a normally distributed random variate,
@@ -1191,7 +1191,7 @@ namespace scythe {
 			 * \param variance The variance of the distribution.
 			 * \param above The upper truncation point of the distribution.
 			 * \param iter The number of iterations to run the slice sampler.
-			 * 
+			 *
 			 * \see rtnorm(double mean, double variance, double below, double above)
 			 * \see rtnorm_combo(double mean, double variance, double below, double above)
 			 * \see rtbnorm_slice(double mean, double variance, double below, unsigned int iter = 10)
@@ -1202,7 +1202,7 @@ namespace scythe {
        * \throw scythe_invalid_arg (Level 1)
        */
       double
-      rtanorm_combo (double mean, double variance, double above, 
+      rtanorm_combo (double mean, double variance, double above,
           const unsigned int iter = 10)
       {
         SCYTHE_CHECK_10(variance <= 0, scythe_invalid_arg,
@@ -1210,7 +1210,7 @@ namespace scythe {
 
         double s = std::sqrt(variance);
         // do rejection sampling and return value
-        if ((mean/s - above/s ) < 0.5){ 
+        if ((mean/s - above/s ) < 0.5){
           double x = rnorm(mean, s);
           while (x > above)
             x = rnorm(mean,s);
@@ -1226,28 +1226,29 @@ namespace scythe {
           double newmu = -1*mean;
           double z = 0;
           double x = below + .00001;
-             
+
           for (unsigned int i=0; i<iter; ++i){
             z = runif() * std::exp(-1 * std::pow((x-newmu), 2)
                 /(2 * variance));
-            x = runif() 
+            x = runif()
               * ((newmu + std::sqrt(-2 * variance * std::log(z)))
                   - below) + below;
           }
-          if (! finite(x)) {
-            Rprintf("Mean extremely far from truncation point. Returning truncation point");
-            return above; 
+          if (! R_finite(x)) {
+            SCYTHE_WARN("Mean extremely far from truncation point. "
+                << "Returning truncation point");
+            return above;
           }
           return -1*x;
         }
       }
 
-      SCYTHE_RNGMETH_MATRIX(rtanorm_combo, double, 
-          SCYTHE_ARGSET(mean, variance, above, iter), double mean, 
+      SCYTHE_RNGMETH_MATRIX(rtanorm_combo, double,
+          SCYTHE_ARGSET(mean, variance, above, iter), double mean,
           double variance, double above, unsigned int iter = 10);
 
       /* Multivariate Distributions */
-      
+
       /*! \brief Generate a Wishart distributed random variate Matrix.
        *
        * This function returns a pseudo-random matrix-valued variate
@@ -1266,14 +1267,14 @@ namespace scythe {
       {
         SCYTHE_CHECK_10(! Sigma.isSquare(), scythe_dimension_error,
             "Sigma not square");
-        SCYTHE_CHECK_10(v < Sigma.rows(), scythe_invalid_arg, 
+        SCYTHE_CHECK_10(v < Sigma.rows(), scythe_invalid_arg,
             "v < Sigma.rows()");
-          
-        Matrix<double,O,Concrete> 
+
+        Matrix<double,O,Concrete>
           A(Sigma.rows(), Sigma.rows());
         Matrix<double,O,Concrete> C = cholesky<O,Concrete>(Sigma);
         Matrix<double,O,Concrete> alpha;
-          
+
         for (unsigned int i = 0; i < v; ++i) {
           alpha = C * rnorm(Sigma.rows(), 1, 0, 1);
           A += (alpha * (t(alpha)));
@@ -1295,14 +1296,14 @@ namespace scythe {
        */
       template <matrix_order O, matrix_style S>
       Matrix<double, O, Concrete>
-      rdirich(const Matrix<double, O, S>& alpha) 
-      { 
+      rdirich(const Matrix<double, O, S>& alpha)
+      {
         // Check for allowable parameters
         SCYTHE_CHECK_10(std::min(alpha) <= 0, scythe_invalid_arg,
             "alpha has elements < 0");
         SCYTHE_CHECK_10(! alpha.isColVector(), scythe_dimension_error,
             "alpha not column vector");
-     
+
         Matrix<double, O, Concrete> y(alpha.rows(), 1);
         double ysum = 0;
 
@@ -1310,8 +1311,10 @@ namespace scythe {
         // and wouldn't get inlined.
         const_matrix_forward_iterator<double,O,O,S> ait;
         const_matrix_forward_iterator<double,O,O,S> alast
-          = alpha.template end_f();
-        typename Matrix<double,O,Concrete>::forward_iterator yit 
+	  //  = alpha.template end_f();
+	  // patch by mattblackwell 2018-08-27
+	  = alpha.end_f();
+        typename Matrix<double,O,Concrete>::forward_iterator yit
           = y.begin_f();
         for (ait = alpha.begin_f(); ait != alast; ++ait) {
           *yit = rgamma(*ait, 1);
@@ -1340,9 +1343,9 @@ namespace scythe {
       template <matrix_order PO1, matrix_style PS1,
                 matrix_order PO2, matrix_style PS2>
       Matrix<double, PO1, Concrete>
-      rmvnorm(const Matrix<double, PO1, PS1>& mu, 
+      rmvnorm(const Matrix<double, PO1, PS1>& mu,
               const Matrix<double, PO2, PS2>& sigma)
-      {  
+      {
         unsigned int dim = mu.rows();
         SCYTHE_CHECK_10(! mu.isColVector(), scythe_dimension_error,
             "mu not column vector");
@@ -1350,7 +1353,7 @@ namespace scythe {
             "sigma not square");
         SCYTHE_CHECK_10(sigma.rows() != dim, scythe_conformation_error,
             "mu and sigma not conformable");
-        
+
         return(mu + cholesky(sigma) * rnorm(dim, 1, 0, 1));
       }
 
@@ -1376,7 +1379,7 @@ namespace scythe {
         SCYTHE_CHECK_10(nu <= 0, scythe_invalid_arg,
             "D.O.F parameter nu <= 0");
 
-        result = 
+        result =
           rmvnorm(Matrix<double, O>(sigma.rows(), 1, true, 0), sigma);
         result /= std::sqrt(rchisq(nu) / nu);
         return result;
@@ -1388,7 +1391,7 @@ namespace scythe {
        *
        * Instantiate a random number generator
        */
-      rng() 
+      rng()
         : rnorm_count_ (1) // Initialize the normal counter
       {}
 
@@ -1428,7 +1431,7 @@ namespace scythe {
         } else { // even numbered passes
           rnorm_count_ = 1;
           return x2_;
-        } 
+        }
       }
 
       /* Generate standard gamma variates */
@@ -1468,12 +1471,12 @@ namespace scythe {
             }
           }
         }
-        
+
         return (accept_);
       }
 
   };
 
-  
-} // end namespace scythe    
+
+} // end namespace scythe
 #endif /* RNG_H */
